package com.chinapnr.android.adapay_android.network;

import android.os.Handler;
import android.os.Message;

import com.alibaba.fastjson.JSON;
import com.chinapnr.android.adapay_android.beans.respbean.BaseRespBean;


import java.lang.ref.WeakReference;
import java.lang.reflect.ParameterizedType;

import okhttp3.Response;

/**
 * @author yumin.chang
 * function 将返回的内容转为 BaseRespBean 的子类
 * create 2019/2/1
 * update
 */
@SuppressWarnings(value = "unchecked")
public abstract class PayNetWorkCallback<T extends BaseRespBean> implements NetworkCallback<T> {
    private Handler responseHandler;

    protected PayNetWorkCallback() {
        responseHandler = new ResposeHandler(this);
    }

    /**
     * 将返回内容转为泛型对应的类型
     *
     * @param response 网络请求返回
     */
    void transferObjType(Response response) {
        ParameterizedType type = (ParameterizedType) this.getClass()
                .getGenericSuperclass();
        Class<T> tClass = (Class<T>) type.getActualTypeArguments()[0];

        T respObj;
        try {

            if (response.body() == null) {
                onNetWorkError("返回数据为空");
                return;
            }
            String resultStr = response.body().string();
            respObj = JSON.parseObject(resultStr, tClass);
        } catch (Exception e) {
            e.printStackTrace();
            onNetWorkError("数据解析错误");
            return;
        }

        if (respObj == null) {
            onNetWorkError("返回内容为空");
            return;
        }

        //如果code为200表示获取配置成功
        if (response.code() == 200) {

            sendResponseMessage(ResposeHandler.RESPONSE_SUCCESS_TAG, respObj);

        } else {
            //code不为200则获取失败
            sendResponseMessage(ResposeHandler.RESPONSE_FAILED_OBJ_TAG, respObj);

        }

    }

    /**
     * 网络错误导致请求失败
     *
     * @param errorMsg 错误原因
     */
    void onNetWorkError(String errorMsg) {
        sendResponseMessage(ResposeHandler.RESPONSE_FAILED_STR_TAG, errorMsg);
    }

    /**
     * 发送网络返回的结果到主线程
     * 由于okhttp的请求回调也是在子线程中完成，
     * 所以要用handler将消息发送到主线程再进行之后的处理
     *
     * @param resultTag 网络请求成功或者失败的标识
     * @param msgObj    请求返回的内容
     */
    private void sendResponseMessage(int resultTag, Object msgObj) {
        Message message = new Message();
        message.what = resultTag;
        message.obj = msgObj;
        responseHandler.sendMessage(message);
    }

    private static class ResposeHandler<T extends BaseRespBean> extends Handler {
        static final int RESPONSE_SUCCESS_TAG = 0x0000;
        static final int RESPONSE_FAILED_STR_TAG = 0x0001;
        static final int RESPONSE_FAILED_OBJ_TAG = 0x0002;

        private WeakReference<PayNetWorkCallback<T>> mOuter;

        private ResposeHandler(PayNetWorkCallback<T> callback) {
            mOuter = new WeakReference<>(callback);
        }

        @Override
        public void handleMessage(Message msg) {
            super.handleMessage(msg);

            PayNetWorkCallback outer = mOuter.get();
            if (outer != null) {

                switch (msg.what) {
                    case RESPONSE_SUCCESS_TAG:
                        //获取成功
                        T successData = (T) msg.obj;
                        outer.onSuccess(successData);
                        break;
                    case RESPONSE_FAILED_OBJ_TAG:
                        //获取失败
                        T failedData = (T) msg.obj;
                        outer.onFail(failedData);
                        break;

                    case RESPONSE_FAILED_STR_TAG:
                        //获取失败
                        String failedReason = (String) msg.obj;
                        outer.onFail(failedReason);
                        break;
                    default:
                        outer.onFail("网络错误，请稍后再试");
                        break;
                }
            }
        }
    }
}