package com.chinapnr.android.adapay_android.network;

import android.support.annotation.NonNull;
import android.text.TextUtils;
import android.util.Log;

import com.alibaba.fastjson.JSON;
import com.chinapnr.android.adapay_android.beans.requestbean.BaseReqBean;

import java.io.IOException;
import java.net.URLEncoder;
import java.util.Map;

import okhttp3.Call;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;


/**
 * @author yumin.chang
 * function 针对于 realme 接口的请求参数处理
 * create 2018/9/5
 * update
 * @version ${VERSION}
 */
@SuppressWarnings("unused")
public class PayRequest {

    private static final String POST = "POST";
    private static final String GET = "GET";

    private String url;
    private PayNetWorkCallback networkCallback;//网络回调的请求
    private OkHttpClient httpClient;
    private BaseReqBean baseReqBean;

    private PayRequest(RequestBuilder builder) {
        this.url = builder.url;
        this.networkCallback = builder.networkCallback;
        this.httpClient = builder.httpClient;
        this.baseReqBean = builder.baseReqBean;
    }

    public Call post() {
        return doRequest(POST);
    }

    public Call get() {
        return doRequest(GET);
    }

    /**
     * 发送请求
     *
     * @param requestMethod 请求方式
     * @return 本次请求索引
     */
    private Call doRequest(String requestMethod) {

        if (networkCallback == null) {
            Log.w("demo", "networkCallback can't be null!");
            return null;
        }
        if (TextUtils.isEmpty(url)) {
            Log.w("demo", "url can not be empty!");
            return null;
        }

        //如果外部不传请求参数，默认请求参数为空
        if (baseReqBean == null) {
            baseReqBean = new BaseReqBean();
        }

        String jsonString = JSON.toJSONString(baseReqBean);

        Request request;

        //根据不同请求方式对参数做不同处理
        switch (requestMethod) {
            case POST:
                request = buildPostRequest(jsonString);
                break;
            case GET:
                request = buildGetRequest(jsonString);
                break;
            default:
                request = buildPostRequest(jsonString);
        }

        //创建/Call
        Call call = httpClient.newCall(request);
        //加入队列 异步操作
        call.enqueue(new okhttp3.Callback() {
            @Override
            public void onResponse(@NonNull Call call, @NonNull Response response) {
                if (response.body() == null) {
                    networkCallback.onNetWorkError("返回信息为空");
                    return;
                }

                networkCallback.transferObjType(response);
            }

            @Override
            public void onFailure(@NonNull Call call, @NonNull IOException e) {
                networkCallback.onNetWorkError(e.getMessage());
            }
        });
        return call;
    }

    /**
     * 处理 get 请求的参数拼接
     *
     * @return request 请求对象
     */
    private Request buildGetRequest(String jsonString) {

        Map<String, Object> paramsMap = JSON.parseObject(jsonString);
        //拼接 get 请求 url
        StringBuilder stringBuilder = new StringBuilder(url);
        stringBuilder.append("?");

        for (Map.Entry<String, Object> entry : paramsMap.entrySet()) {

            String value = "";
            try {
                value = URLEncoder.encode((String) entry.getValue(), "utf-8");
            } catch (Exception e) {
                e.printStackTrace();
            }

            String paramsPairStr = entry.getKey() + "=" + value + "&";
            stringBuilder.append(paramsPairStr);
        }

        return new Request.Builder()
                .url(stringBuilder.toString())
                .addHeader("charset", "UTF-8")
                .get()
                .build();
    }

    /**
     * 处理 post 请求的参数拼接
     *
     * @return request 请求对象
     */
    private Request buildPostRequest(String jsonString) {
        //以表单形式发送，处理参数
        MediaType json = MediaType.parse("application/json; charset=utf-8");
        RequestBody requestBody = RequestBody.create(json, jsonString);

        return new Request.Builder()
                .url(url)//请求的url
                .addHeader("charset", "UTF-8")
                .post(requestBody)
                .build();
    }

    /**
     * 网络请求构造器
     */
    @SuppressWarnings("unused")
    public static class RequestBuilder {

        private String url;
        private PayNetWorkCallback networkCallback;//网络回调的请求
        private OkHttpClient httpClient;
        private BaseReqBean baseReqBean;
        private int timeout;

        public RequestBuilder() {
        }

        public RequestBuilder setUrl(String url) {
            this.url = url;
            return this;
        }

        public RequestBuilder setReqBean(BaseReqBean baseReqBean) {
            this.baseReqBean = baseReqBean;
            return this;
        }

        public RequestBuilder setNetWorkCallback(PayNetWorkCallback networkCallback) {
            this.networkCallback = networkCallback;
            return this;
        }

        public RequestBuilder setTimeout(int timeout) {
            this.timeout = timeout;
            return this;
        }

        public PayRequest build() {
            if (timeout <= 0 || timeout > 60) {
                timeout = 30;
            }
            this.httpClient = HttpManager.getInstance().getHttpClient(timeout);

            return new PayRequest(this);
        }
    }
}
